﻿using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using Syncfusion.Windows.Controls.Grid;
using Syncfusion.Windows.Controls.Scroll;
using System;

namespace ComboBoxCell
{
     /// <summary>
    /// A cell model for the <see cref="ComboBoxCellRenderer"/>.
    /// </summary>
    public class ComboBoxCellModel : GridCellModel<ComboBoxCellRenderer>
    {
    }

    /// <summary>
    /// A renderer that manages a ComboBox specified with <see cref="TreeRenderStyleInfo.CellTemplateKey"/>
    /// of the <see cref="TreeRenderStyleInfo"/> inside cells. The <see cref="ContentControl.Content"/>
    /// will be the  <see cref="TreeRenderStyleInfo.CellValue"/> which binds the ComboBox to the value
    /// of the style.
    /// </summary>
    public class ComboBoxCellRenderer : GridVirtualizingCellRenderer<ComboBox>
    {
        public ComboBoxCellRenderer()
        {
            AllowRecycle = true;
            IsControlTextShown = false; // let combobox deal directly with CellValue
            IsFocusable = true;
        }

        /// <summary>
        /// Called to initialize the content of the cell
        /// using the information from the cell style (value, text,
        /// behavior etc.). You must override this method in your
        /// derived class.
        /// </summary>
        /// <param name="comboBox">The UI element.</param>
        /// <param name="style">The cell style info.</param>
        public override void OnInitializeContent(ComboBox comboBox, GridRenderStyleInfo style)
        {
            comboBox.VerticalContentAlignment = VerticalAlignment.Center;
            comboBox.Padding = new Thickness(4, 0, 0,0);
            //comboBox.IsEditable = true;
            comboBox.ItemsSource = Window1.employeesSource.Employees;
            comboBox.DisplayMemberPath = "Name";
            comboBox.SelectedValuePath = "";
            comboBox.SelectedValue = this.GetControlValue(style);
        }

        /// <summary>
        /// Gets the control value from editor.
        /// </summary>
        /// <param name="uiElement">The UI element.</param>
        /// <returns></returns>
        public override object GetControlValueFromEditor(ComboBox uiElement)
        {
            return uiElement.SelectedValue;
        }

        /// <summary>
        /// Wire events from comboBox
        /// </summary>
        /// <param name="comboBox"></param>
        protected override void OnWireUIElement(ComboBox comboBox)
        {
            base.OnWireUIElement(comboBox);

            comboBox.SelectionChanged += new SelectionChangedEventHandler(comboBox_SelectionChanged);
            comboBox.DropDownOpened += new System.EventHandler(comboBox_DropDownOpened);
            comboBox.DropDownClosed += new System.EventHandler(comboBox_DropDownClosed);
        }

        /// <summary>
        /// Unwire previously wired events from comboBox. 
        /// </summary>
        /// <param name="comboBox"></param>
        protected override void OnUnwireUIElement(ComboBox comboBox)
        {
            base.OnUnwireUIElement(comboBox);

            comboBox.SelectionChanged -= new SelectionChangedEventHandler(comboBox_SelectionChanged);
            comboBox.DropDownOpened -= new System.EventHandler(comboBox_DropDownOpened);
            comboBox.DropDownClosed -= new System.EventHandler(comboBox_DropDownClosed);
        }

        void comboBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ComboBox comboBox = (ComboBox)sender;
            if (!this.IsInArrange && IsCurrentCell(comboBox))
            {
                Console.WriteLine(comboBox.SelectedValue);
                if (!SetControlValue(comboBox.SelectedValue))
                    RefreshContent(); // reverses change.
            }
        }

        void comboBox_DropDownOpened(object sender, System.EventArgs e)
        {
            GridControl.MouseControllerDispatcher.SuspendMouse();
        }

        void comboBox_DropDownClosed(object sender, System.EventArgs e)
        {
            GridControl.MouseControllerDispatcher.ResumeMouse();
        }

        protected override void OnGridPreviewTextInput(TextCompositionEventArgs e)
        {
            CurrentCell.ScrollInView();
            CurrentCell.BeginEdit(true);
            GridControl.InvalidateCell(CurrentCell.CellRowColumnIndex);
            GridControl.InvalidateVisual(false);
        }

        protected override bool ShouldGridTryToHandlePreviewKeyDown(KeyEventArgs e)
        {
            // return false to indicate the CurrentCellUIElement should handle the key
            // and the grid should ignore it.
            bool isControlKey = (e.KeyboardDevice.Modifiers & ModifierKeys.Control) != ModifierKeys.None;
            bool isAltKey = (e.KeyboardDevice.Modifiers & ModifierKeys.Alt) != ModifierKeys.None;
            if (isControlKey)
                return true;

            if (CurrentCellUIElement != null)
            {
                switch (e.Key)
                {
                    case Key.Tab:
                        return true;

                    case Key.Up:
                    case Key.Down:
                    case Key.Right:
                    case Key.Left:
                        if (isAltKey)
                        {
                            CurrentCell.BeginEdit(true);
                            return false;
                        }

                        return !IsFocused;

                    case Key.End:
                    case Key.Home:
                        {
                            CurrentCell.BeginEdit(true);
                            return false;
                        }
                }
            }

            return base.ShouldGridTryToHandlePreviewKeyDown(e);
        }


    }

}
